"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotionDetector = void 0;
const loader_1 = require("./coco/loader");
const image_utils_1 = require("../utils/image-utils");
const google_photos_1 = require("../utils/google-photos");
class MotionDetector {
    constructor(api, config, mqtt, unifiFlows, cameras, log) {
        this.api = api;
        this.config = config;
        this.unifiConfig = config.unifi;
        this.googlePhotosConfig = config.googlePhotos;
        this.flows = unifiFlows;
        this.cameras = cameras;
        this.log = log;
        this.modelLoader = new loader_1.Loader(log);
        this.detector = null;
        const userStoragePath = this.api.user.storagePath();
        image_utils_1.ImageUtils.userStoragePath = userStoragePath;
        this.gPhotos = config.upload_gphotos && this.googlePhotosConfig ? new google_photos_1.GooglePhotos(config.googlePhotos, userStoragePath, log) : null;
        this.mqtt = mqtt;
    }
    async setupMotionChecking(configuredAccessories) {
        this.configuredAccessories = configuredAccessories;
        let intervalFunction;
        if (this.unifiConfig.enhanced_motion) {
            this.detector = await this.modelLoader.loadCoco();
            intervalFunction = this.checkMotionEnhanced.bind(this);
        }
        else {
            intervalFunction = this.checkMotion.bind(this);
        }
        setInterval(() => {
            try {
                intervalFunction();
            }
            catch (error) {
                this.log.debug('Error during motion interval loop: ' + error);
            }
        }, this.unifiConfig.motion_interval);
        return;
    }
    async checkMotion() {
        try {
            await this.flows.assignMotionEventsToCameras(this.cameras);
        }
        catch (error) {
            this.log.debug('Cannot get latest motion info: ' + error);
        }
        outer: for (const configuredAccessory of this.configuredAccessories) {
            configuredAccessory.getService(this.api.hap.Service.MotionSensor).setCharacteristic(this.api.hap.Characteristic.MotionDetected, 0);
            if (!configuredAccessory.context.motionEnabled) {
                continue;
            }
            for (const camera of this.cameras) {
                if (configuredAccessory.context.id === camera.id) {
                    camera.lastDetectionSnapshot = null;
                    if (!camera.lastMotionEvent || this.isSkippableLongRunningMotion(configuredAccessory, camera.lastMotionEvent)) {
                        continue outer;
                    }
                    this.log.info('Motion detected (' + camera.lastMotionEvent.score + '%) by camera ' + camera.name + ' !!!!');
                    configuredAccessory.getService(this.api.hap.Service.MotionSensor).setCharacteristic(this.api.hap.Characteristic.MotionDetected, 1);
                    this.mqtt.sendMessageOnTopic(JSON.stringify({ score: camera.lastMotionEvent.score, timestamp: new Date().toISOString(), snapshot: image_utils_1.ImageUtils.resizeCanvas(camera.lastDetectionSnapshot, 480, 270).toBuffer('image/jpeg', { quality: 0.5 }).toString('base64') }), camera.name);
                    let snapshot;
                    try {
                        snapshot = await image_utils_1.ImageUtils.createImage('http://' + camera.ip + '/snap.jpeg');
                        camera.lastDetectionSnapshot = await this.persistSnapshot(snapshot, 'Motion detected (' + camera.lastMotionEvent.score + '%) by camera ' + camera.name, []);
                    }
                    catch (error) {
                        this.log.debug('Cannot save snapshot: ' + error);
                    }
                    continue outer;
                }
            }
        }
    }
    async checkMotionEnhanced() {
        try {
            await this.flows.assignMotionEventsToCameras(this.cameras);
        }
        catch (error) {
            this.log.debug('Cannot get latest motion info: ' + error);
        }
        outer: for (const configuredAccessory of this.configuredAccessories) {
            configuredAccessory.getService(this.api.hap.Service.MotionSensor).setCharacteristic(this.api.hap.Characteristic.MotionDetected, 0);
            if (!configuredAccessory.context.motionEnabled) {
                continue;
            }
            for (const camera of this.cameras) {
                if (configuredAccessory.context.id === camera.id) {
                    camera.lastDetectionSnapshot = null;
                    if (!camera.lastMotionEvent) {
                        continue outer;
                    }
                    let snapshot;
                    try {
                        snapshot = await image_utils_1.ImageUtils.createImage('http://' + camera.ip + '/snap.jpeg');
                    }
                    catch (error) {
                        continue outer;
                    }
                    const detections = await this.detector.detect(snapshot, this.unifiConfig.debug);
                    for (const classToDetect of this.unifiConfig.enhanced_classes) {
                        const detection = this.getDetectionForClassName(classToDetect, detections);
                        if (detection) {
                            if (this.isSkippableLongRunningMotion(configuredAccessory, camera.lastMotionEvent)) {
                                continue outer;
                            }
                            const score = Math.round(detection.score * 100);
                            if (score >= this.unifiConfig.enhanced_motion_score) {
                                this.log.info('Detected: ' + detection.class + ' (' + score + '%) by camera ' + camera.name);
                                camera.lastDetectionSnapshot = await this.persistSnapshot(snapshot, detection.class + ' detected (' + score + '%) by camera ' + camera.name, [detection]);
                                configuredAccessory.getService(this.api.hap.Service.MotionSensor).setCharacteristic(this.api.hap.Characteristic.MotionDetected, 1);
                                this.mqtt.sendMessageOnTopic(JSON.stringify({ class: detection.class, score, timestamp: new Date().toISOString(), snapshot: image_utils_1.ImageUtils.resizeCanvas(camera.lastDetectionSnapshot, 480, 270).toBuffer('image/jpeg', { quality: 0.5 }).toString('base64') }), camera.name);
                                continue outer;
                            }
                            else {
                                this.log.debug('Detected class: ' + detection.class + ' rejected due to score: ' + score + '% (must be ' + this.unifiConfig.enhanced_motion_score + '% or higher)');
                            }
                        }
                        else {
                            this.log.debug('None of the required classes found by enhanced motion detection, discarding!');
                        }
                    }
                }
            }
        }
    }
    async persistSnapshot(snapshot, description, detections) {
        try {
            let annotatedImage = await image_utils_1.ImageUtils.generateAnnotatedImage(snapshot, detections);
            if ((this.unifiConfig.save_snapshot || this.config.upload_gphotos) && annotatedImage) {
                const fileLocation = await image_utils_1.ImageUtils.saveCanvasToFile(annotatedImage);
                this.log.debug('The snapshot has been saved to: ' + fileLocation);
                if (this.config.upload_gphotos) {
                    const fileName = fileLocation.split('/').pop();
                    this.gPhotos
                        .uploadImage(fileLocation, fileName, description)
                        .then((url) => {
                        if (url) {
                            this.log.debug('Photo uploaded: ' + url);
                        }
                        else {
                            this.log.debug('Photo not uploaded!');
                        }
                        if (!this.unifiConfig.save_snapshot) {
                            image_utils_1.ImageUtils.remove(fileLocation);
                            this.log.debug('The snapshot has been removed from: ' + fileLocation);
                        }
                    });
                }
            }
            return annotatedImage;
        }
        catch (error) {
            this.log.debug('Error persisting snapshot image: ' + error);
        }
    }
    isSkippableLongRunningMotion(accessory, motionEvent) {
        if (this.unifiConfig.motion_repeat_interval) {
            if (accessory.context.lastMotionId === motionEvent.id) {
                accessory.context.lastMotionIdRepeatCount++;
                if (accessory.context.lastMotionIdRepeatCount === (this.unifiConfig.motion_repeat_interval / this.unifiConfig.motion_interval)) {
                    accessory.context.lastMotionIdRepeatCount = 0;
                }
                else {
                    this.log.debug('Motion detected inside of skippable timeframe, ignoring!');
                    return true;
                }
            }
            else {
                accessory.context.lastMotionId = motionEvent.id;
                accessory.context.lastMotionIdRepeatCount = 0;
            }
        }
        return false;
    }
    getDetectionForClassName(className, detections) {
        for (const detection of detections) {
            if (detection.class.toLowerCase() === className.toLowerCase()) {
                return detection;
            }
        }
        return null;
    }
}
exports.MotionDetector = MotionDetector;
//# sourceMappingURL=motion.js.map