"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnifiProtectMotionPlatform = void 0;
const unifi_1 = require("./unifi/unifi");
const unifi_flows_1 = require("./unifi/unifi-flows");
const settings_1 = require("./settings");
const motion_1 = require("./motion/motion");
const unifi_camera_accessory_info_1 = require("./characteristics/unifi-camera-accessory-info");
const unifi_camera_motion_sensor_1 = require("./characteristics/unifi-camera-motion-sensor");
const unifi_camera_doorbell_1 = require("./characteristics/unifi-camera-doorbell");
const unifi_camera_streaming_1 = require("./streaming/unifi-camera-streaming");
const unifi_streaming_delegate_1 = require("./streaming/unifi-streaming-delegate");
const mqtt_1 = require("./utils/mqtt");
class UnifiProtectMotionPlatform {
    constructor(log, config, api) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.hap = this.api.hap;
        this.Accessory = this.api.platformAccessory;
        this.accessories = [];
        if (!config || !this.config.unifi) {
            this.log.info('Incorrect plugin configuration!');
            return;
        }
        this.config.unifi.excluded_cameras = this.config.unifi.excluded_cameras ? this.config.unifi.excluded_cameras : [];
        this.api.on("didFinishLaunching", () => {
            setTimeout(async () => {
                try {
                    const unifi = new unifi_1.Unifi(this.config.unifi, 500, 2, this.log);
                    this.uFlows = new unifi_flows_1.UnifiFlows(unifi, this.config.unifi, await unifi_1.Unifi.determineEndpointStyle(this.config.unifi.controller, this.log), this.log);
                    unifi_streaming_delegate_1.UnifiStreamingDelegate.uFlows = this.uFlows;
                    await this.didFinishLaunching();
                }
                catch (error) {
                    this.log.error(error);
                }
            });
        });
    }
    async didFinishLaunching() {
        let cameras = [];
        try {
            cameras = await this.filterValidCameras();
        }
        catch (error) {
            this.log.info('Cannot get cameras: ' + error);
        }
        cameras
            .map((camera) => {
            this.log.info(JSON.stringify(camera, null, 4));
            camera.uuid = this.hap.uuid.generate(camera.id);
            const accessory = this.accessories.find((existingAccessory) => existingAccessory.UUID === camera.uuid);
            return { camera, accessory };
        })
            .filter((cameraAndAccessory) => {
            return cameraAndAccessory.accessory === undefined;
        })
            .map((cameraAndAccessory) => {
            return cameraAndAccessory.camera;
        })
            .forEach((camera) => {
            const cameraAccessory = new this.Accessory(camera.name, camera.uuid);
            cameraAccessory.context.cameraConfig = {
                uuid: camera.uuid,
                name: camera.name,
                camera: camera
            };
            unifi_camera_accessory_info_1.UnifiCameraAccessoryInfo.createAccessoryInfo(camera, cameraAccessory, this.hap);
            this.log.info('Adding ' + cameraAccessory.context.cameraConfig.uuid + ' (' + cameraAccessory.context.cameraConfig.name + ')');
            this.configureAccessory(cameraAccessory);
            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [cameraAccessory]);
        });
        this.accessories = this.accessories.filter((accessory) => {
            if (!cameras.find((x) => x.uuid === accessory.context.cameraConfig.uuid)) {
                this.log.info('Removing ' + accessory.context.cameraConfig.uuid + ' (' + accessory.context.cameraConfig.name + ')');
                this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
            else {
                return accessory;
            }
        });
        try {
            this.mqtt = new mqtt_1.Mqtt(this.config.mqtt_enabled === true ? this.config.mqtt : null, this.log);
            this.accessories.forEach((accessory) => {
                const cameraConfig = accessory.context.cameraConfig;
                cameraConfig.camera = cameras.find((cam) => cam.id === accessory.context.cameraConfig.camera.id);
                unifi_camera_motion_sensor_1.UnifiCameraMotionSensor.setupMotionSensor(cameraConfig, accessory, this.config, this.mqtt, this.hap, this.log);
                unifi_camera_doorbell_1.UnifiCameraDoorbell.setupDoorbell(cameraConfig, accessory, this.config, this.hap, this.log);
                unifi_camera_streaming_1.UnifiCameraStreaming.setupStreaming(cameraConfig, accessory, this.config, this.api, this.log);
            });
            const motionDetector = new motion_1.MotionDetector(this.api, this.config, this.mqtt, this.uFlows, cameras, this.log);
            await motionDetector.setupMotionChecking(this.accessories);
            this.log.info('Motion checking setup done!');
        }
        catch (error) {
            this.log.info('Error during motion checking setup: ' + error);
        }
    }
    configureAccessory(cameraAccessory) {
        this.log.info('Configuring accessory ' + cameraAccessory.displayName);
        cameraAccessory.on("identify", () => {
            this.log.info(cameraAccessory.displayName + ' identified!');
        });
        this.accessories.push(cameraAccessory);
    }
    async filterValidCameras() {
        return (await this.uFlows.enumerateCameras()).filter((camera) => {
            if (!this.config.unifi.excluded_cameras.includes(camera.id) && camera.streams.length >= 1) {
                return camera;
            }
            else {
                if (this.config.unifi.excluded_cameras.includes(camera.id)) {
                    this.log.info('Camera (' + camera.name + ') excluded by config!');
                }
                else if (camera.streams.length < 1) {
                    this.log.info('Camera (' + camera.name + ') excluded because is has no available RTSP stream!');
                }
            }
        });
    }
}
exports.UnifiProtectMotionPlatform = UnifiProtectMotionPlatform;
//# sourceMappingURL=unifi-protect-motion-platform.js.map