"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Unifi = void 0;
const utils_1 = require("../utils/utils");
const node_fetch_1 = require("node-fetch");
class Unifi {
    constructor(config, initialBackoffDelay, maxRetries, log) {
        this.config = config;
        this.initialBackoffDelay = initialBackoffDelay;
        this.maxRetries = maxRetries;
        this.log = log;
        this.networkLogger = this.config.debug_network_traffic ? this.log : undefined;
    }
    static async determineEndpointStyle(baseControllerUrl, log) {
        if (baseControllerUrl && baseControllerUrl.endsWith('/')) {
            throw new Error('Controller URL should NOT end with a slash!');
        }
        const headers = new node_fetch_1.Headers();
        headers.set('Content-Type', 'application/json');
        const response = await utils_1.Utils.fetch(baseControllerUrl, { method: 'GET' }, headers);
        const csrfToken = response.headers.get('X-CSRF-Token');
        if (csrfToken) {
            log.info('Endpoint Style: UnifiOS');
            return {
                authURL: baseControllerUrl + '/api/auth/login',
                apiURL: baseControllerUrl + '/proxy/protect/api',
                isUnifiOS: true,
                csrfToken: csrfToken
            };
        }
        else {
            log.info('Endpoint Style: Unifi Protect (Legacy)');
            return {
                authURL: baseControllerUrl + '/api/auth',
                apiURL: baseControllerUrl + '/api',
                isUnifiOS: false
            };
        }
    }
    async authenticate(username, password, endpointStyle) {
        if (!username || !password) {
            throw new Error('Username and password should be filled in!');
        }
        const headers = new node_fetch_1.Headers();
        headers.set('Content-Type', 'application/json');
        if (endpointStyle.isUnifiOS) {
            headers.set('X-CSRF-Token', endpointStyle.csrfToken);
        }
        const loginPromise = utils_1.Utils.fetch(endpointStyle.authURL, {
            body: JSON.stringify({ username: username, password: password }),
            method: 'POST'
        }, headers, this.networkLogger);
        const response = await utils_1.Utils.retry(this.maxRetries, () => { return loginPromise; }, this.initialBackoffDelay);
        this.log.debug('Authenticated, returning session');
        if (endpointStyle.isUnifiOS) {
            endpointStyle.csrfToken = response.headers.get('X-CSRF-Token');
            return {
                cookie: response.headers.get('Set-Cookie'),
                timestamp: Date.now()
            };
        }
        else {
            return {
                authorization: response.headers.get('Authorization'),
                timestamp: Date.now()
            };
        }
    }
    isSessionStillValid(session, endpointStyle) {
        if (session) {
            if ((session.timestamp + ((endpointStyle.isUnifiOS ? 1 : 12) * 3600 * 1000)) >= Date.now()) {
                return true;
            }
            else {
                this.log.debug('WARNING: Session expired, a new session must be created!');
            }
        }
        else {
            this.log.debug('WARNING: No previous session found, a new session must be created!');
        }
        return false;
    }
    async enumerateMotionCameras(session, endPointStyle) {
        const headers = new node_fetch_1.Headers();
        headers.set('Content-Type', 'application/json');
        if (endPointStyle.isUnifiOS) {
            headers.set('Cookie', session.cookie);
            headers.set('X-CSRF-Token', endPointStyle.csrfToken);
        }
        else {
            headers.set('Authorization', 'Bearer ' + session.authorization);
        }
        const bootstrapPromise = utils_1.Utils.fetch(endPointStyle.apiURL + '/bootstrap', { method: 'GET' }, headers, this.networkLogger);
        const response = await utils_1.Utils.retry(this.maxRetries, () => { return bootstrapPromise; }, this.initialBackoffDelay);
        const cams = (await response.json()).cameras;
        this.log.debug('Cameras retrieved, enumerating motion sensors');
        return cams.map((cam) => {
            if (this.config.debug) {
                this.log.debug(cam);
            }
            const streams = [];
            for (const channel of cam.channels) {
                if (channel.rtspAlias) {
                    streams.push({
                        name: channel.name,
                        alias: channel.rtspAlias,
                        width: channel.width,
                        height: channel.height,
                        fps: channel.fps
                    });
                }
            }
            streams.sort((a, b) => {
                return (a.height * a.width) - (b.height * b.width);
            });
            this.log.info('Found camera: ' + cam.name + ' (id: ' + cam.id + ')');
            return {
                id: cam.id,
                name: cam.name,
                ip: cam.host,
                mac: cam.mac,
                type: cam.type,
                firmware: cam.firmwareVersion,
                streams: streams,
                supportsTwoWayAudio: cam.hasSpeaker && cam.speakerSettings.isEnabled,
                talkbackSettings: cam.talkbackSettings
            };
        });
    }
    async getMotionEvents(session, endPointStyle) {
        const endEpoch = Date.now();
        const startEpoch = endEpoch - (this.config.motion_interval * 2);
        const headers = new node_fetch_1.Headers();
        headers.set('Content-Type', 'application/json');
        if (endPointStyle.isUnifiOS) {
            headers.set('Cookie', session.cookie);
            headers.set('X-CSRF-Token', endPointStyle.csrfToken);
        }
        else {
            headers.set('Authorization', 'Bearer ' + session.authorization);
        }
        const eventsPromise = utils_1.Utils.fetch(endPointStyle.apiURL + '/events?end=' + endEpoch + '&start=' + startEpoch + '&type=motion', { method: 'GET' }, headers, this.networkLogger);
        const response = await utils_1.Utils.retry(this.maxRetries, () => { return eventsPromise; }, this.initialBackoffDelay);
        const events = await response.json();
        return events.map((event) => {
            if (this.config.debug) {
                this.log.debug(event);
            }
            return {
                id: event.id,
                cameraId: event.camera,
                camera: null,
                score: event.score,
                timestamp: event.start
            };
        });
    }
    async getSnapshotForCamera(session, endPointStyle, camera, width, height) {
        const headers = new node_fetch_1.Headers();
        headers.set('Content-Type', 'application/json');
        if (endPointStyle.isUnifiOS) {
            headers.set('Cookie', session.cookie);
            headers.set('X-CSRF-Token', endPointStyle.csrfToken);
        }
        else {
            headers.set('Authorization', 'Bearer ' + session.authorization);
        }
        const params = new URLSearchParams({ force: "true", width: width, height: height });
        const response = await utils_1.Utils.fetch(endPointStyle.apiURL + '/cameras/' + camera.id + '/snapshot/?' + params, {
            method: 'GET'
        }, headers, this.networkLogger);
        if (!(response === null || response === void 0 ? void 0 : response.ok)) {
            this.log.debug(JSON.stringify(response, null, 4));
            throw new Error('Could not get snapshot for ' + camera.name);
        }
        return response.buffer();
    }
    static generateStreamingUrlForBestMatchingResolution(baseSourceUrl, streams, requestedWidth, requestedHeight) {
        const targetResolution = requestedWidth * requestedHeight;
        const selectedAlias = streams
            .map(((stream) => {
            return {
                resolution: stream.width * stream.height,
                alias: stream.alias
            };
        }))
            .filter((data) => {
            return data.resolution <= targetResolution;
        })
            .sort((a, b) => {
            return b.resolution - a.resolution;
        })
            .shift().alias;
        return baseSourceUrl + selectedAlias;
    }
}
exports.Unifi = Unifi;
//# sourceMappingURL=unifi.js.map